/* 
 * tkWinInit.c --
 *
 *	This file contains Windows-specific interpreter initialization
 *	functions.
 *
 * Copyright (c) 1995 Sun Microsystems, Inc.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

static char sccsid[] = "@(#) tkWinInit.c 1.4 95/09/15 11:45:55";

#include "tkWinInt.h"
#include <winreg.h>


/*
 *----------------------------------------------------------------------
 *
 * TkPlatformInit --
 *
 *	Performs Windows-specific interpreter initialization related to the
 *      tk_library variable.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	Sets "tk_library" Tcl variable
 *
 *----------------------------------------------------------------------
 */

void
TkPlatformInit(interp)
    Tcl_Interp *interp;
{
    char *ptr;
    HKEY regKey;
    DWORD size = MAX_PATH;
    Tcl_DString ds;
    Tcl_DStringInit(&ds);

    /*
     * Look in the environment and the registry for a TK_LIBRARY
     * entry.  Store the result in the dynamic string.
     */

    ptr = Tcl_GetVar2(interp, "env", "TK_LIBRARY", TCL_GLOBAL_ONLY);
    if (ptr == NULL) {
	Tcl_DStringAppend(&ds, "Software\\SUNW\\Tk\\", -1);
	Tcl_DStringAppend(&ds, TK_VERSION, -1);

	/*
	 * Windows 3.1 only supports HKEY_CLASSES_ROOT.
	 */

	if(RegOpenKeyEx(HKEY_LOCAL_MACHINE, Tcl_DStringValue(&ds), 0,
		KEY_READ, &regKey) != ERROR_SUCCESS) {
	    RegOpenKeyEx(HKEY_CLASSES_ROOT, Tcl_DStringValue(&ds), 0,
		    KEY_READ, &regKey);
	}
	Tcl_DStringSetLength(&ds, size);

	/*
	 * Since Win32s doesn't support multiple valued keys, we have to try
	 * using the name as a value name and as a subkey.
	 */

	if((RegQueryValueEx(regKey, "TK_LIBRARY", NULL, NULL,
		Tcl_DStringValue(&ds), &size) == ERROR_SUCCESS)
		|| (RegQueryValue(regKey, "TK_LIBRARY", Tcl_DStringValue(&ds),
			&size) == ERROR_SUCCESS)) {
	    RegCloseKey(regKey);
	    Tcl_DStringSetLength(&ds, size);
	} else {
	    Tcl_DStringSetLength(&ds, 0);
	}
    } else {
	Tcl_DStringSetLength(&ds, 0);
	Tcl_DStringAppend(&ds, ptr, -1);
    }

    /*
     * If the path doesn't exist, look for the library relative to the
     * tcl.dll library.
     */

    if ((Tcl_DStringLength(&ds) == 0)
	    || (access(Tcl_DStringValue(&ds), 0) != 0)) {
	char *lastSlash = NULL;
	char libName[13];	/* used to hold constructed library name */

	/*
	 * Path was invalid, so we need to construct a default path from the
	 * location of the executable.  Convert backslashes at the same time.
	 */

	Tcl_DStringSetLength(&ds, MAX_PATH);
	sprintf(libName, "tk%d%d.dll", TK_MAJOR_VERSION, TK_MINOR_VERSION);
	Tcl_DStringSetLength(&ds, SearchPath(NULL, libName, NULL, MAX_PATH,
		Tcl_DStringValue(&ds), &lastSlash));
	if (Tcl_DStringLength(&ds) > 0) {
	    Tcl_DStringSetLength(&ds, lastSlash - Tcl_DStringValue(&ds));
	    Tcl_DStringAppend(&ds, "../lib/tk", -1);
	    Tcl_DStringAppend(&ds, TK_VERSION, -1);
	}
    }

    /*
     * Convert backslashes to forward slashes in path names.
     */

    for (ptr = Tcl_DStringValue(&ds); *ptr != '\0'; ptr++) {
	if (*ptr == '\\') {
	    *ptr = '/';
	}
    }

    Tcl_SetVar(interp, "tk_library", Tcl_DStringValue(&ds), TCL_GLOBAL_ONLY);
    Tcl_DStringFree(&ds);
}
